@@ -11,12 +11,12 @@
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
      Affero General Public License for more details.
-    
+
      You should have received a copy of the GNU Affero General Public License
      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
      SPDX-License-Identifier: AGPL3.0-or-later
-*/
+ */
 
 /**
  * @author Christian Grothoff
@@ -34,10 +34,11 @@
 #ifndef GNUNET_GNSRECORD_LIB_H
 #define GNUNET_GNSRECORD_LIB_H
 
+#include "gnunet_identity_service.h"
+
 #ifdef __cplusplus
-extern "C"
-{
-#if 0                           /* keep Emacsens' auto-indent happy */
+extern "C" {
+#if 0 /* keep Emacsens' auto-indent happy */
 }
 #endif
 #endif
@@ -56,7 +57,7 @@ extern "C"
 /**
  * Record type for GNS zone transfer ("PKEY").
  */
-#define GNUNET_GNSRECORD_TYPE_PKEY 65536
+#define GNUNET_GNSRECORD_TYPE_PKEY GNUNET_IDENTITY_TYPE_ECDSA
 
 /**
  * Record type for GNS nick names ("NICK").
@@ -94,29 +95,27 @@ extern "C"
 #define GNUNET_GNSRECORD_TYPE_PHONE 65543
 
 /**
- * Record type for identity attributes (of IDENTITY).
+ * Record type for identity attributes (of RECLAIM).
  */
-#define GNUNET_GNSRECORD_TYPE_ID_ATTR 65544
+#define GNUNET_GNSRECORD_TYPE_RECLAIM_ATTRIBUTE 65544
 
 /**
- * Record type for an identity token (of IDENTITY-TOKEN).
+ * Record type for local ticket references
  */
-#define GNUNET_GNSRECORD_TYPE_ID_TOKEN 65545
+#define GNUNET_GNSRECORD_TYPE_RECLAIM_TICKET 65545
 
 /**
- * Record type for the private metadata of an identity token (of IDENTITY-TOKEN).
+ * 65546 reserved
  */
-#define GNUNET_GNSRECORD_TYPE_ID_TOKEN_METADATA 65546
 
 /**
- * Record type for credential
+ * 65547 deleted, free to use
  */
-#define GNUNET_GNSRECORD_TYPE_CREDENTIAL 65547
 
 /**
  * Record type for policies
  */
-#define GNUNET_GNSRECORD_TYPE_POLICY 65548
+#define GNUNET_GNSRECORD_TYPE_DELEGATE 65548
 
 /**
  * Record type for reverse lookups
@@ -124,14 +123,14 @@ extern "C"
 #define GNUNET_GNSRECORD_TYPE_ATTRIBUTE 65549
 
 /**
- * Record type for ABE records
+ * Record type for reclaim records
  */
-#define GNUNET_GNSRECORD_TYPE_ABE_KEY 65550
+#define GNUNET_GNSRECORD_TYPE_RECLAIM_ATTRIBUTE_REF 65550
 
 /**
- * Record type for ABE master keys
+ * Record type for RECLAIM master
  */
-#define GNUNET_GNSRECORD_TYPE_ABE_MASTER 65551
+#define GNUNET_GNSRECORD_TYPE_RECLAIM_MASTER 65551
 
 /**
  * Record type for reclaim OIDC clients
@@ -143,12 +142,28 @@ extern "C"
  */
 #define GNUNET_GNSRECORD_TYPE_RECLAIM_OIDC_REDIRECT 65553
 
+/**
+ * Record type for an attribute attestation
+ */
+#define GNUNET_GNSRECORD_TYPE_RECLAIM_CREDENTIAL 65554
+
+/**
+ * Record type for a presentation of a credential (used
+ * in a ticket record set)
+ */
+#define GNUNET_GNSRECORD_TYPE_RECLAIM_PRESENTATION 65555
+
+/**
+ * Record type for EDKEY delegations.
+ */
+#define GNUNET_GNSRECORD_TYPE_EDKEY GNUNET_IDENTITY_TYPE_EDDSA
+
+
 /**
  * Flags that can be set for a record.
  */
 enum GNUNET_GNSRECORD_Flags
 {
-
   /**
    * No special options.
    */
@@ -161,10 +176,9 @@ enum GNUNET_GNSRECORD_Flags
   GNUNET_GNSRECORD_RF_PRIVATE = 2,
 
   /**
-   * This flag is currently unused; former RF_PENDING flag
-   *
-   * GNUNET_GNSRECORD_RF_UNUSED = 4,
+   * This is a supplemental record.
    */
+  GNUNET_GNSRECORD_RF_SUPPLEMENTAL = 4,
 
   /**
    * This expiration time of the record is a relative
@@ -178,17 +192,17 @@ enum GNUNET_GNSRECORD_Flags
    */
   GNUNET_GNSRECORD_RF_SHADOW_RECORD = 16
 
-  /**
-   * When comparing flags for record equality for removal,
-   * which flags should must match (in addition to the type,
-   * name, expiration value and data of the record)?  All flags
-   * that are not listed here will be ignored for this purpose.
-   * (for example, we don't expect that users will remember to
-   * pass the '--private' option when removing a record from
-   * the namestore, hence we don't require this particular option
-   * to match upon removal).  See also
-   * #GNUNET_GNSRECORD_records_cmp.
-   */
+/**
+ * When comparing flags for record equality for removal,
+ * which flags should must match (in addition to the type,
+ * name, expiration value and data of the record)?  All flags
+ * that are not listed here will be ignored for this purpose.
+ * (for example, we don't expect that users will remember to
+ * pass the '--private' option when removing a record from
+ * the namestore, hence we don't require this particular option
+ * to match upon removal).  See also
+ * #GNUNET_GNSRECORD_records_cmp.
+ */
 #define GNUNET_GNSRECORD_RF_RCMP_FLAGS (GNUNET_GNSRECORD_RF_RELATIVE_EXPIRATION)
 };
 
@@ -198,7 +212,6 @@ enum GNUNET_GNSRECORD_Flags
  */
 struct GNUNET_GNSRECORD_Data
 {
-
   /**
    * Binary value stored in the DNS record.  Note: "data" must never
    * be individually 'malloc'ed, but instead always points into some
@@ -255,23 +268,21 @@ struct GNUNET_GNSRECORD_PlaceData
   /* Followed by struct GNUNET_PeerIdentity relays[relay_count] */
 };
 
-
 /**
  * Information we have in an encrypted block with record data (i.e. in the DHT).
  */
-struct GNUNET_GNSRECORD_Block
+struct GNUNET_GNSRECORD_EcdsaBlock
 {
+  /**
+   * Derived key used for signing; hash of this is the query.
+   */
+  struct GNUNET_CRYPTO_EcdsaPublicKey derived_key;
 
   /**
    * Signature of the block.
    */
   struct GNUNET_CRYPTO_EcdsaSignature signature;
 
-  /**
-   * Derived key used for signing; hash of this is the query.
-   */
-  struct GNUNET_CRYPTO_EcdsaPublicKey derived_key;
-
   /**
    * Number of bytes signed; also specifies the number of bytes
    * of encrypted data that follow.
@@ -286,6 +297,17 @@ struct GNUNET_GNSRECORD_Block
   /* followed by encrypted data */
 };
 
+struct GNUNET_GNSRECORD_Block
+{
+  uint32_t type;
+
+  union
+  {
+    struct GNUNET_GNSRECORD_EcdsaBlock ecdsa_block;
+    //struct GNUNET_GNSRECORD_EddsaBlock eddsa_block;
+  };
+};
+
 
 /**
  * Record type used to box up SRV and TLSA records.  For example, a
@@ -299,7 +321,6 @@ struct GNUNET_GNSRECORD_Block
  */
 struct GNUNET_GNSRECORD_BoxRecord
 {
-
   /**
    * Protocol of the boxed record (6 = TCP, 17 = UDP, etc.).
    * Yes, in IP protocols are usually limited to 8 bits. In NBO.
@@ -317,7 +338,6 @@ struct GNUNET_GNSRECORD_BoxRecord
   uint32_t record_type GNUNET_PACKED;
 
   /* followed by the 'original' record */
-
 };
 
 /**
@@ -332,7 +352,7 @@ struct GNUNET_GNSRECORD_ReverseRecord
   /**
    * The public key of the namespace the is delegating to our namespace
    */
-  struct GNUNET_CRYPTO_EcdsaPublicKey pkey;
+  struct GNUNET_IDENTITY_PublicKey pkey;
 
   /**
    * The expiration time of the delegation
@@ -352,11 +372,8 @@ GNUNET_NETWORK_STRUCT_END
  * @param rd_count number of entries in @a rd array
  * @param rd array of records with data to store
  */
-typedef void
-(*GNUNET_GNSRECORD_RecordCallback) (void *cls,
-                                    unsigned int rd_count,
-                                    const struct GNUNET_GNSRECORD_Data *rd);
-
+typedef void (*GNUNET_GNSRECORD_RecordCallback) (
+  void *cls, unsigned int rd_count, const struct GNUNET_GNSRECORD_Data *rd);
 
 
 /* ***************** API related to GNSRECORD plugins ************** */
@@ -371,9 +388,8 @@ typedef void
  * @return NULL on error, otherwise human-readable representation of the value
  */
 char *
-GNUNET_GNSRECORD_value_to_string (uint32_t type,
-				  const void *data,
-				  size_t data_size);
+GNUNET_GNSRECORD_value_to_string (uint32_t type, const void *data,
+                                  size_t data_size);
 
 
 /**
@@ -387,10 +403,8 @@ GNUNET_GNSRECORD_value_to_string (uint32_t type,
  * @return #GNUNET_OK on success
  */
 int
-GNUNET_GNSRECORD_string_to_value (uint32_t type,
-				  const char *s,
-				  void **data,
-				  size_t *data_size);
+GNUNET_GNSRECORD_string_to_value (uint32_t type, const char *s, void **data,
+                                  size_t *data_size);
 
 
 /**
@@ -425,7 +439,7 @@ GNUNET_GNSRECORD_number_to_typename (uint32_t type);
  */
 ssize_t
 GNUNET_GNSRECORD_records_get_size (unsigned int rd_count,
-				   const struct GNUNET_GNSRECORD_Data *rd);
+                                   const struct GNUNET_GNSRECORD_Data *rd);
 
 
 /**
@@ -439,9 +453,8 @@ GNUNET_GNSRECORD_records_get_size (unsigned int rd_count,
  */
 ssize_t
 GNUNET_GNSRECORD_records_serialize (unsigned int rd_count,
-				    const struct GNUNET_GNSRECORD_Data *rd,
-				    size_t dest_size,
-				    char *dest);
+                                    const struct GNUNET_GNSRECORD_Data *rd,
+                                    size_t dest_size, char *dest);
 
 
 /**
@@ -454,16 +467,14 @@ GNUNET_GNSRECORD_records_serialize (unsigned int rd_count,
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  */
 int
-GNUNET_GNSRECORD_records_deserialize (size_t len,
-				      const char *src,
-				      unsigned int rd_count,
-				      struct GNUNET_GNSRECORD_Data *dest);
+GNUNET_GNSRECORD_records_deserialize (size_t len, const char *src,
+                                      unsigned int rd_count,
+                                      struct GNUNET_GNSRECORD_Data *dest);
 
 
 /* ******* general APIs relating to blocks, records and labels ******** */
 
 
-
 /**
  * Test if a given record is expired.
  *
@@ -490,10 +501,11 @@ GNUNET_GNSRECORD_string_to_lowercase (const char *src);
  * NOT reentrant!
  *
  * @param z public key of a zone
- * @return string form; will be overwritten by next call to #GNUNET_GNSRECORD_z2s.
+ * @return string form; will be overwritten by next call to
+ * #GNUNET_GNSRECORD_z2s.
  */
 const char *
-GNUNET_GNSRECORD_z2s (const struct GNUNET_CRYPTO_EcdsaPublicKey *z);
+GNUNET_GNSRECORD_z2s (const struct GNUNET_IDENTITY_PublicKey *z);
 
 
 /**
@@ -507,7 +519,7 @@ GNUNET_GNSRECORD_z2s (const struct GNUNET_CRYPTO_EcdsaPublicKey *z);
  *         key in an encoding suitable for DNS labels.
  */
 const char *
-GNUNET_GNSRECORD_pkey_to_zkey (const struct GNUNET_CRYPTO_EcdsaPublicKey *pkey);
+GNUNET_GNSRECORD_pkey_to_zkey (const struct GNUNET_IDENTITY_PublicKey *pkey);
 
 
 /**
@@ -521,7 +533,7 @@ GNUNET_GNSRECORD_pkey_to_zkey (const struct GNUNET_CRYPTO_EcdsaPublicKey *pkey);
  */
 int
 GNUNET_GNSRECORD_zkey_to_pkey (const char *zkey,
-			       struct GNUNET_CRYPTO_EcdsaPublicKey *pkey);
+                               struct GNUNET_IDENTITY_PublicKey *pkey);
 
 
 /**
@@ -532,9 +544,9 @@ GNUNET_GNSRECORD_zkey_to_pkey (const char *zkey,
  * @param query hash to use for the query
  */
 void
-GNUNET_GNSRECORD_query_from_private_key (const struct GNUNET_CRYPTO_EcdsaPrivateKey *zone,
-					 const char *label,
-					 struct GNUNET_HashCode *query);
+GNUNET_GNSRECORD_query_from_private_key (
+  const struct GNUNET_IDENTITY_PrivateKey *zone, const char *label,
+  struct GNUNET_HashCode *query);
 
 
 /**
@@ -545,9 +557,9 @@ GNUNET_GNSRECORD_query_from_private_key (const struct GNUNET_CRYPTO_EcdsaPrivate
  * @param query hash to use for the query
  */
 void
-GNUNET_GNSRECORD_query_from_public_key (const struct GNUNET_CRYPTO_EcdsaPublicKey *pub,
-					const char *label,
-					struct GNUNET_HashCode *query);
+GNUNET_GNSRECORD_query_from_public_key (
+  const struct GNUNET_IDENTITY_PublicKey *pub, const char *label,
+  struct GNUNET_HashCode *query);
 
 
 /**
@@ -560,11 +572,11 @@ GNUNET_GNSRECORD_query_from_public_key (const struct GNUNET_CRYPTO_EcdsaPublicKe
  * @param rd_count number of records in @a rd
  */
 struct GNUNET_GNSRECORD_Block *
-GNUNET_GNSRECORD_block_create (const struct GNUNET_CRYPTO_EcdsaPrivateKey *key,
-			       struct GNUNET_TIME_Absolute expire,
-			       const char *label,
-			       const struct GNUNET_GNSRECORD_Data *rd,
-			       unsigned int rd_count);
+GNUNET_GNSRECORD_block_create (const struct GNUNET_IDENTITY_PrivateKey *key,
+                               struct GNUNET_TIME_Absolute expire,
+                               const char *label,
+                               const struct GNUNET_GNSRECORD_Data *rd,
+                               unsigned int rd_count);
 
 
 /**
@@ -579,7 +591,7 @@ GNUNET_GNSRECORD_block_create (const struct GNUNET_CRYPTO_EcdsaPrivateKey *key,
  * @param rd_count number of records in @a rd
  */
 struct GNUNET_GNSRECORD_Block *
-GNUNET_GNSRECORD_block_create2 (const struct GNUNET_CRYPTO_EcdsaPrivateKey *key,
+GNUNET_GNSRECORD_block_create2 (const struct GNUNET_IDENTITY_PrivateKey *key,
                                 struct GNUNET_TIME_Absolute expire,
                                 const char *label,
                                 const struct GNUNET_GNSRECORD_Data *rd,
@@ -609,11 +621,10 @@ GNUNET_GNSRECORD_block_verify (const struct GNUNET_GNSRECORD_Block *block);
  *        not well-formed
  */
 int
-GNUNET_GNSRECORD_block_decrypt (const struct GNUNET_GNSRECORD_Block *block,
-				const struct GNUNET_CRYPTO_EcdsaPublicKey *zone_key,
-				const char *label,
-				GNUNET_GNSRECORD_RecordCallback proc,
-				void *proc_cls);
+GNUNET_GNSRECORD_block_decrypt (
+  const struct GNUNET_GNSRECORD_Block *block,
+  const struct GNUNET_IDENTITY_PublicKey *zone_key, const char *label,
+  GNUNET_GNSRECORD_RecordCallback proc, void *proc_cls);
 
 
 /**
@@ -638,11 +649,87 @@ GNUNET_GNSRECORD_records_cmp (const struct GNUNET_GNSRECORD_Data *a,
  * @return absolute expiration time
  */
 struct GNUNET_TIME_Absolute
-GNUNET_GNSRECORD_record_get_expiration_time (unsigned int rd_count,
-					     const struct GNUNET_GNSRECORD_Data *rd);
+GNUNET_GNSRECORD_record_get_expiration_time (
+  unsigned int rd_count, const struct GNUNET_GNSRECORD_Data *rd);
 
 
-#if 0                           /* keep Emacsens' auto-indent happy */
+/**
+ * Returns the length of this block in bytes.
+ * Block length strongly depends on the zone type.
+ *
+ * @param block the block.
+ * @return the length of this block in bytes
+ */
+size_t
+GNUNET_GNSRECORD_block_get_size (const struct GNUNET_GNSRECORD_Block *block);
+
+/**
+ * Returns the expiration of a block.
+ *
+ * @param block the block.
+ * @return the block expiration.
+ */
+struct GNUNET_TIME_Absolute
+GNUNET_GNSRECORD_block_get_expiration (const struct GNUNET_GNSRECORD_Block *block);
+
+
+/**
+ * Builds the query hash from a block.
+ *
+ * @param block the block.
+ * @param query where to write the query hash.
+ * @return GNUNET_SYSERR on error.
+ */
+enum GNUNET_GenericReturnValue
+GNUNET_GNSRECORD_query_from_block (const struct GNUNET_GNSRECORD_Block *block,
+                                   struct GNUNET_HashCode *query);
+
+
+/**
+ * Build a #GNUNET_GNSRECORD_PublicKey from
+ * zone delegation resource record data.
+ *
+ * @param data the record data-
+ * @param data_size the data size.
+ * @param type the record type
+ * @param key the identity key to store the data in (must be allocated).
+ * @return GNUNET_OK if successful.
+ */
+enum GNUNET_GenericReturnValue
+GNUNET_GNSRECORD_identity_from_data (const char *data,
+                                     size_t data_size,
+                                     uint32_t type,
+                                     struct GNUNET_IDENTITY_PublicKey *key);
+
+
+/**
+ * Create record data and size from an identity key.
+ *
+ * @param key the identity key to use.
+ * @param data the record data (will be allocated)
+ * @param data_size the allocated data size.
+ * @param type the resulting record type
+ * @return GNUNET_OK if successful.
+ */
+enum GNUNET_GenericReturnValue
+GNUNET_GNSRECORD_data_from_identity (const struct GNUNET_IDENTITY_PublicKey *key,
+                                     char **data,
+                                     size_t *data_size,
+                                     uint32_t *type);
+
+
+/**
+ * Check if this type is one of the supported GNS zone
+ * types.
+ *
+ * @param type the type to check
+ * @return GNUNET_YES if it is one of the supported types.
+ */
+enum GNUNET_GenericReturnValue
+GNUNET_GNSRECORD_is_zonekey_type (uint32_t type);
+
+
+#if 0 /* keep Emacsens' auto-indent happy */
 {
 #endif
 #ifdef __cplusplus
@@ -651,4 +738,4 @@ GNUNET_GNSRECORD_record_get_expiration_time (unsigned int rd_count,
 
 #endif
 
-/** @} */  /* end of group */
+/** @} */ /* end of group */
